#include "mod_papi_private.h"

typedef struct {
        char *data;
        size_t size;
} papi_post_data_t;

typedef struct {
        request_rec *r;
        int cors_flag;
} papi_header_data_t;

static size_t papi_post_read_data (void *ptr, size_t size, size_t nmemb, void *data)
{
        size_t realsize = size * nmemb;
        papi_post_data_t *mem = (papi_post_data_t *)data;
                
        mem->data = (char *)realloc(mem->data, mem->size + realsize + 1);
        if (mem->data) {
                memcpy(&(mem->data[mem->size]), ptr, realsize);
                mem->size += realsize;
                mem->data[mem->size] = 0;
        }
        return realsize;
}

// SIUV - HMR pass headers from the curl request
//static size_t papi_post_read_header(char* header_data, size_t size, size_t nitems, void *userdata)
static size_t papi_post_read_header(char* header_data, size_t size, size_t nitems, void *userdata)
{
//        request_rec *r= (request_rec *)userdata;

        papi_header_data_t *udata=(papi_header_data_t *)userdata;
        request_rec *r=udata->r;
        int cors_flag=udata->cors_flag;
        
        size_t realsize = size * nitems;      // size is always 1
        
        char *header = apr_pcalloc (r->pool,  realsize + 1);
        memcpy(header, header_data, realsize);
        if( realsize>0  ) header[realsize]='\0';    // need ending null!
        if( realsize>=2 ) header[realsize-2]='\0';  // strip ending "\r\n"

        // APACHE_LOG (APLOG_DEBUG, "CURL POST REQUEST: header returned [%s]",header);
        
        if( realsize >2 ){
          char *ptr = strchr (header, ':');
          if( ptr ){  // Has ':' (name: value)
            *ptr = '\0';                   // truncate at ":"
            ptr++;
            if ( *(ptr) == ' ' ) ptr++;     // skip next space
            APACHE_LOG (APLOG_DEBUG, "CURL POST REQUEST: header returned %s :: %s",header,ptr);
             
            if( strcmp(header,"Date") != 0 &&
                strcmp(header,"Content-Length") != 0 &&
                strcmp(header,"Keep-Alive") != 0 &&
                strcmp(header,"Connection") != 0 &&
                strcmp(header,"Transfer-Encoding") != 0 &&   // CRITICO!
                // strncmp(header,"Access-Control",strlen("Access-Control")) != 0 &&
                // strcmp(header,"Set-Cookie") != 0 &&
                // strcmp(header,"Content-Type") != 0 &&
                strcmp(header,"Server") != 0
                ){
                // Solo deja pasar las cabeceras CORS si está puesto "PAPISetCORSHeaders off"
                if( strncmp(header,"Access-Control",strlen("Access-Control")) != 0 ){  
                  APACHE_LOG (APLOG_DEBUG, "CURL POST REQUEST: header returned PASSED %s :: %s",header,ptr);
                  apr_table_add(r->headers_out,header,ptr);
                  }
                else if ( !cors_flag ){
                  APACHE_LOG (APLOG_DEBUG, "CURL POST ALLOW REQUEST: header returned PASSED %s :: %s",header,ptr);
                  apr_table_add(r->headers_out,header,ptr);
                  }
              }
            }
          }

        return realsize;
}

int papi_post_handler (request_rec *r, papi_dir_config *d, const char *post)
{
        char *base_uri = ap_construct_url (r->pool, r->unparsed_uri, r);
        char *ptr = strchr (base_uri, '?');
        if (ptr) *ptr = '\0';
        
        /* SIUV-HMR - que ponga los argumentos del URL! */
        char *uri;
        if (strcmp(r->args,"") == 0) uri=base_uri;
        else uri= apr_psprintf (r->pool,"%s?%s",base_uri,r->args);

        papi_post_data_t *papi_post_data = apr_pcalloc (r->pool, sizeof (papi_post_data_t));

        papi_header_data_t *papi_header_data = apr_pcalloc (r->pool, sizeof (papi_post_data_t));
        papi_header_data->r=r;
        papi_header_data->cors_flag= d->set_cors_headers ;
        
        curl_global_init (CURL_GLOBAL_ALL);
        CURL *curl = curl_easy_init();
        curl_easy_setopt (curl, CURLOPT_URL, uri);
        curl_easy_setopt (curl, CURLOPT_HEADERFUNCTION, papi_post_read_header); // SIUV - HMR
//        curl_easy_setopt (curl, CURLOPT_HEADERDATA, (void *)r );                // SIUV - HMR
        curl_easy_setopt (curl, CURLOPT_HEADERDATA, (void *)papi_header_data );   // SIUV - HMR
        curl_easy_setopt (curl, CURLOPT_POST, 1);
        curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, papi_post_read_data);
        curl_easy_setopt (curl, CURLOPT_WRITEDATA, (void *)papi_post_data);

        /* SIUV - HMR complica mucho el setup (hay que configurar los certificados
           en la libreria CURL) y el cliente ya lo ha verificado! */
        curl_easy_setopt (curl, CURLOPT_SSL_VERIFYHOST, 0);
        curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, 0);
        /* SIUV - HMR: admitir que la respuesta este comprimida con mod-deflate! */
        curl_easy_setopt(curl,CURLOPT_ENCODING , "");
        
        // Copy saved header in request.
        int i;
        const apr_array_header_t *headers_in = apr_table_elts (r->headers_in);
        struct curl_slist *headers_out = NULL;
        for (i=0; i < headers_in->nelts; i++) {
                apr_table_entry_t *entry = ((apr_table_entry_t *)headers_in->elts)+i;
                char *header;
                if (!strcmp (entry->key, "Content-Length")) {
                        header = apr_psprintf (r->pool, "Content-Length: %d", (int)strlen(post));
                } else if (!strcmp (entry->key, "Accept-Encoding")) {
                        header = apr_psprintf (r->pool, "Accept-Encoding: identity");
                } else {
                        header = apr_pstrcat (r->pool, entry->key, ": ", entry->val, NULL);
                }
                headers_out = curl_slist_append (headers_out, header);
                APACHE_LOG (APLOG_DEBUG, "CURL POST REQUEST: header [%s]",header);
        }
        
        
        APACHE_LOG (APLOG_DEBUG, "CURL POST REQUEST: url [%s] body [%s][%ld]",uri,post,strlen(post));
        curl_easy_setopt (curl, CURLOPT_HTTPHEADER, headers_out);
        curl_easy_setopt (curl, CURLOPT_POSTFIELDS, post);
        curl_easy_setopt (curl, CURLOPT_POSTFIELDSIZE, strlen(post));
        int res = curl_easy_perform (curl);
        if (res == CURLE_OK) {
                char *ct;
                curl_easy_getinfo (curl, CURLINFO_CONTENT_TYPE, &ct);
                ap_set_content_type (r, apr_pstrdup (r->pool, ct));
        } else { /* SIUV -HMR - agnadida comprobacion de error! */
          APACHE_LOG (APLOG_ERR, "ERROR (%d) in POST CURL subrequest",res);
          return HTTP_BAD_REQUEST; 
          }
        
        curl_easy_cleanup (curl);
        curl_global_cleanup();
        
        APACHE_LOG (APLOG_DEBUG, "CURL POST REQUEST: result [%s]",papi_post_data->data);

        ap_rputs (papi_post_data->data, r);

        free (papi_post_data->data);
        
        return DONE;
}

int papi_read_body (request_rec *r, char **buffer)
{
        char buf[MAX_SIZE+1];
        size_t bytes, count = 0;
        
        *buffer = apr_pstrdup (r->pool, "");
        
        if (ap_setup_client_block (r, REQUEST_CHUNKED_DECHUNK) != OK) {
                APACHE_LOG (APLOG_ERR, "Bad request body!");
                return HTTP_BAD_REQUEST;
        }
        
        if (ap_should_client_block (r)) {
                for (bytes = ap_get_client_block (r, buf, MAX_SIZE); 
                         bytes > 0;
                         bytes = ap_get_client_block (r, buf, MAX_SIZE)) {
                  /* SIUV HMR- corregido puntero */
                  buf[bytes] = '\0';
                  *buffer = apr_pstrcat (r->pool, *buffer, buf, NULL);
                  count += bytes;
                }
                /* SIUV HMR- corregido puntero */
                (*buffer)[count] = '\0'; 
  APACHE_LOG (APLOG_WARNING, "BODY read %ld bytes.",count);
        } else {
                APACHE_LOG (APLOG_WARNING, "No request body.");
        }

        return OK;
}

